summaryrefslogtreecommitdiff
path: root/src/runtime/asm_loong64.s
diff options
context:
space:
mode:
Diffstat (limited to 'src/runtime/asm_loong64.s')
-rw-r--r--src/runtime/asm_loong64.s32
1 files changed, 28 insertions, 4 deletions
diff --git a/src/runtime/asm_loong64.s b/src/runtime/asm_loong64.s
index 6029dbc8c3..b93ad3316d 100644
--- a/src/runtime/asm_loong64.s
+++ b/src/runtime/asm_loong64.s
@@ -460,13 +460,23 @@ g0:
TEXT ·cgocallback(SB),NOSPLIT,$24-24
NO_LOCAL_POINTERS
+ // Skip cgocallbackg, just dropm when fn is nil, and frame is the saved g.
+ // It is used to dropm while thread is exiting.
+ MOVV fn+0(FP), R5
+ BNE R5, loadg
+ // Restore the g from frame.
+ MOVV frame+8(FP), g
+ JMP dropm
+
+loadg:
// Load m and g from thread-local storage.
MOVB runtime·iscgo(SB), R19
BEQ R19, nocgo
JAL runtime·load_g(SB)
nocgo:
// If g is nil, Go did not create the current thread.
// Call needm to obtain one for temporary use.
// In this case, we're running on the thread stack, so there's
// lots of space, but the linker doesn't know. Hide the call from
@@ -479,7 +489,7 @@ nocgo:
needm:
MOVV g, savedm-8(SP) // g is zero, so is m.
- MOVV $runtime·needm(SB), R4
+ MOVV $runtime·needAndBindM(SB), R4
JAL (R4)
// Set m->sched.sp = SP, so that if a panic happens
@@ -551,10 +561,24 @@ havem:
MOVV savedsp-24(SP), R13 // must match frame size
MOVV R13, (g_sched+gobuf_sp)(g)
// If the m on entry was nil, we called needm above to borrow an m
// for the duration of the call. Since the call is over, return it with dropm.
MOVV savedm-8(SP), R12
BNE R12, droppedm
+
+ // Skip dropm to reuse it in the next call, when a pthread key has been created.
+ MOVV _cgo_pthread_key_created(SB), R12
+ // It means cgo is disabled when _cgo_pthread_key_created is a nil pointer, need dropm.
+ BEQ R12, dropm
+ MOVV (R12), R12
+ BNE R12, droppedm
+
+dropm:
MOVV $runtime·dropm(SB), R4
JAL (R4)
droppedm:
@@ -460,13 +460,23 @@ g0:
JAL runtime·load_g(SB)
nocgo:
- // If g is nil, Go did not create the current thread.
+ // If g is nil, Go did not create the current thread,
+ // or if this thread never called into Go on pthread platforms.
// Call needm to obtain one for temporary use.
// In this case, we're running on the thread stack, so there's
@@ -551,10 +561,24 @@ havem:
MOVV savedsp-24(SP), R13 // must match frame size
MOVV R13, (g_sched+gobuf_sp)(g)
- // If the m on entry was nil, we called needm above to borrow an m
- // for the duration of the call. Since the call is over, return it with dropm.
+ // If the m on entry was nil, we called needm above to borrow an m,
+ // 1. for the duration of the call on non-pthread platforms,
+ // 2. or the duration of the C thread alive on pthread platforms.
+ // If the m on entry wasn't nil,
+ // 1. the thread might be a Go thread,
+ // 2. or it's wasn't the first call from a C thread on pthread platforms,
+ // since the we skip dropm to resue the m in the first call.
MOVV savedm-8(SP), R12
BNE R12, droppedm